/*
*  Name:      integer.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type integer
*  Date:      $Date: 2002/09/25 23:13:45 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_INTEGER__
#define _UESQLC_INTEGER__

#include "base_type.hh"


///  Name-space for UESQL.
namespace uesql
{

  ///  SQL type INTEGER.
  class TInteger : public TBaseType
  {

    private:

      ///  Integer value.
      long int   liValue;


    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TInteger (void)  :
        TBaseType ()   ,
        liValue   (0L) {}

      ///  Builds a new instance from another instance.
      TInteger (const TInteger& rktINTEGER) :
        TBaseType ()                        ,
        liValue   (rktINTEGER.liValue)
      {
        setNull (rktINTEGER.isNull());
      }

      TInteger (const long int kliVALUE) :
        TBaseType ()                     ,
        liValue   (kliVALUE)
      {
        setNull (false);
      }

      TInteger (const char* pkcINTEGER) :
        TBaseType ()                    ,
        liValue   (0L)
      {
        set (pkcINTEGER);
      }

      TInteger (const std::string& rkyINTEGER) :
        TBaseType ()                           ,
        liValue   (0L)
      {
        set (rkyINTEGER);
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      TInteger& set (const char* pkcVALUE)
      {
        setNull ( !pkcVALUE || !*pkcVALUE );
        if ( !isNull() )
        {
          liValue = std::atol (pkcVALUE);
        }
        return *this;
      }

      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      TInteger& set (const std::string& rkyVALUE)
      {
        return set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::Format ("%ld", liValue);
        }
        return yResult;
      }

      ///  Returns the long integer value.
      operator long int (void) const
      {
        return liValue;
      }

  };  // class TInteger

}  // namespace uesql


#endif  // not _UESQLC_INTEGER__
