/*
*  Name:      exceptions.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC exceptions
*  Date:      $Date: 2002/09/13 23:50:33 $
*  Revision:  $Revision: 1.13 $
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include "exceptions.hh"


//
//  L O C A L   V A R I A B L E S
//

///  Warning message.
static const char*   _pkcWarningDescription = "";

///  Error message.
static const char*   _pkcErrorDescription   = "";

///  Fatal error message.
static const char*   _pkcFatalDescription   = "uesqlc: ";


//
//  C O N S T R U C T O R S
//

TWarningException::TWarningException ( const char* pkcSPECIFIC_DESCRIPTION ,
                                       const char* pkcFILE_NAME            ,
                                       const int   kiLINE_NUMBER           ) :
  TBaseException          ( _pkcWarningDescription  ,
                            pkcSPECIFIC_DESCRIPTION ,
                            pkcFILE_NAME            ,
                            kiLINE_NUMBER           )                        ,
  mpcl::TWarningException ()                                                 ,
  TException              ()                                                 {}


TErrorException::TErrorException ( const char* pkcSPECIFIC_DESCRIPTION ,
                                   const char* pkcFILE_NAME            ,
                                   const int   kiLINE_NUMBER           ) :
  TBaseException        ( _pkcErrorDescription    ,
                          pkcSPECIFIC_DESCRIPTION ,
                          pkcFILE_NAME            ,
                          kiLINE_NUMBER           )                      ,
  mpcl::TErrorException ()                                               ,
  TException            ()                                               {}


TFatalException::TFatalException ( const char* pkcSPECIFIC_DESCRIPTION ,
                                   const char* pkcFILE_NAME            ,
                                   const int   kiLINE_NUMBER           ) :
  TBaseException        ( _pkcFatalDescription    ,
                          pkcSPECIFIC_DESCRIPTION ,
                          pkcFILE_NAME            ,
                          kiLINE_NUMBER           )                      ,
  mpcl::TFatalException ()                                               ,
  TException            ()                                               {}


//
//  S E L E C T O R S
//

void TException::
write (std::basic_ostream<char>& rtTARGET_OSTREAM) const
{

  if ( !fileName().empty() )
  {
    TBaseException::write (rtTARGET_OSTREAM);
  }
  else
  {
    rtTARGET_OSTREAM << generalDescription();
    if ( !specificDescription().empty() )
    {
      rtTARGET_OSTREAM << specificDescription();
    }
    rtTARGET_OSTREAM << std::endl;
  }
 
}  // write()


void TFatalException::
write (std::basic_ostream<char>& rtTARGET_OSTREAM) const
{

  //
  //  Fatal errors doesn't depend on files.
  //
  rtTARGET_OSTREAM << generalDescription();
  if ( !specificDescription().empty() )
  {
    rtTARGET_OSTREAM << specificDescription();
  }
  rtTARGET_OSTREAM << std::endl;
 
}  // write()
